/******************************************************************************
*       UART单线键盘接口驱动库
*
* 程序说明：BC6xxx和BC759x芯片键盘接口的驱动函数库
* 版权所有: 北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.6
* 初始版本：2021年3月10日
* 版本历史:
*   2021年3月   V1.0
*   2021年3月   V1.1  增加了对长时间无按键的检测支持
*   2021年3月   V1.2  改进执行效率
*   2021年3月   V1.3  修正数组定义中的错误
*   2021年3月   V1.4  修改数据类型从unsigned char至uint8_t等
*   2021年4月   V1.5  增加了LastKeyEvent的初始化值
*   2021年4月   V1.6  重新改写了格式以兼容C89等较早编译器标准
*   2023年4月   V1.7  更改Keyboard初始化代码，更好支持C89编译器
* 使用方法：
*   将此头文件key_scan.h加入所有需要使用本驱动库中函数的c源文件，并将本文件加入项目源文件列表，
*   即可从用户程序中直接调用本库中函数。
*	通常可以在串口中断中呼叫update_key_status()函数，然后从主程序循环中查询 is_key_changed()
*	函数，如果结果不为零，则呼叫get_key_value()函数获取键值，根据键值执行相应键盘响应操作即可。
*	详细使用方法，参见《UART单线键盘接口驱动库技术说明书》
******************************************************************************/

#include "key_scan.h"

#ifndef NULL
#define NULL 0
#endif

/* 定义键盘接口数据结构并设置默认值。此处设置为不检测按键释放，长按键时长为2000次计数 */
struct keyboard_t
{
    uint8_t         NewKeyAvailable;
    uint8_t         DetectKeyRelease;
    uint8_t         LastKeyEvent;
    uint8_t         ReportedKey;
};

struct keyboard_t Keyboard = {
    0,          /* NewKeyAvailable */
    0,          /* DetectKeyRelease */
    0xff,       /* LastKeyEvent */
    0,          /* ReportedKey */
};

/* 各种键数下对应的位 */
const uint8_t FullBits[8] = { 0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f };

/******************************************************************************
* 查询是否有新按键
* 根据工作模式的设置，返回是否当前有未读取的新按键。只有符合当前工作模式设置的键盘变化才会返回，
* 比如设置了不检测按键的释放，则按键释放不会改变这个查询的结果。
* 返回值: 如果没有新按键返回0,其它值表示有新按键
******************************************************************************/
uint8_t is_key_changed(void)
{
    return Keyboard.NewKeyAvailable;
}
/******************************************************************************
* 更新键盘状态
* 输入参数为UART所接收数据，此函数一般可在UART中断处理中呼叫，但也可以根据需要从任何地方呼叫。
* 参数: RxData  --  UART从BC6xxx或BC759x所接收到的数据
******************************************************************************/
void update_key_status(uint8_t RxData)
{
    Keyboard.LastKeyEvent = RxData; /* 保存最新按键事件 */
    if (Keyboard.DetectKeyRelease || !(Keyboard.LastKeyEvent & 0x80)) /* 如果检测按键释放，或为按键按下，报告新按键 */
    {
        Keyboard.ReportedKey     = Keyboard.LastKeyEvent;
        Keyboard.NewKeyAvailable = 1;
    }

}
/******************************************************************************
* 获取当前键值
* 可在任何时候呼叫。呼叫此函数后，如果当前 is_key_changed() 函数返回非零,呼叫后则返回值会变成
* 零。仅返回当前工作模式下有效的键值，即如果设置为不检测按键释放，则仅会返回按键按下所对应的键值。
* 返回值: 键值。如果当前按键是组合键或长按键，则返回用户给组合键和长按键自定义的键值。
******************************************************************************/
uint8_t get_key_value(void)
{
    Keyboard.NewKeyAvailable = 0;
    return Keyboard.ReportedKey;
}