#ifndef KEY_SCAN_H
#define KEY_SCAN_H

/******************************************************************************
*       UART单线键盘接口驱动库
*
* 程序说明：BC6xxx和BC759x芯片键盘接口的驱动函数库
* 版权所有: 北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.6
* 初始版本：2021年3月10日
* 版本历史:
*   2021年3月   V1.0
*   2021年3月   V1.1  增加了对长时间无按键的检测支持
*   2021年3月   V1.2  改进执行效率
*   2021年3月   V1.3  修正数组定义中的错误
*   2021年3月   V1.4  修改数据类型从unsigned char至uint8_t等
*   2021年4月   V1.5  增加了LastKeyEvent的初始化值
*   2021年4月   V1.6  重新改写了格式以兼容C89等较早编译器标准
*   2023年4月   V1.7  更改Keyboard初始化代码，更好支持C89编译器
* 使用方法：
*   将此头文件加入所有需要使用本驱动库中函数的c源文件，并将key_scan.c文件加入项目源文件列表，
*   即可从用户程序中直接调用本库中函数。
*	通常可以在串口中断中呼叫update_key_status()函数，然后从主程序循环中查询 is_key_changed()
*	函数，如果结果不为零，则呼叫get_key_value()函数获取键值，根据键值执行相应键盘响应操作即可。
*	详细使用方法，参见《UART单线键盘接口驱动库技术说明书》
******************************************************************************/

/*        
stdint.h包含 uint8_t 等数据类型的定义，如果您所用环境为C99标准以下或没有stdint.h 文件，
请自行定义如下：*/
typedef unsigned char   uint8_t;
typedef unsigned short  uint16_t;


/******************************************************************************
* 查询是否有新按键 is_key_changed()
* 根据工作模式的设置，返回是否当前有未读取的新按键。只有符合当前工作模式设置的键盘变化才会返回，
* 比如设置了不检测按键的释放，则按键释放不会改变这个查询的结果。
* 返回值: 如果没有新按键返回0,其它值表示有新按键
******************************************************************************/
uint8_t is_key_changed(void);

/******************************************************************************
* 设置键盘检测模式 set_detect_mode()
* 决定是否检测按键(包括组合键)的释放。
* 参数: Mode  --  工作模式， 0=只检测按键按下(导通), 1=同时检测按键按下和释放
******************************************************************************/
void set_detect_mode(uint8_t Mode);

/******************************************************************************
* 设置长按键计数值 set_longpress_count()
* 长按键的时间由此计数值决定。用户定期呼叫 long_press_tick() 函数，每次呼叫内部计数器被
* 加一，而每次键盘活动则使计数器清零。当内部计数器的值大于此值时，即触发一个长按键检测，如果
* 此前最后一次按键(包括组合键)属于需检测长按键的键值，则报告一个长按键(以用户设定的特殊键值
* 报告一个新按键)
* 参数: CountLimit  --  最大值65534, 达到触发条件所需的计数值，默认值为 2000
******************************************************************************/
void set_longpress_count(uint16_t CountLimit);

/******************************************************************************
* 设置回调函数 set_callback()
* 一般应用时在串口中断中呼叫更新键盘状态函数，而实际的键盘处理程序则放在主循环中。因为处理键盘
* 事件一般不是高优先级的操作，在主程序中处理按键可以减少串口中断占用的处理器时间。当键盘事件
* 需要立刻处理，或者程序为中断驱动型，不存在主循环时，可以设置此回调函数，此函数会在检测到有效
* 键盘事件后自动呼叫。
* 回调函数应该具有一个 uint8_t 类型的输入参数用以接收键值，而返回类型应该为 void. 
* 如果不设置或者设置为 NULL ,则回调函数不会被呼叫。请注意如果 update_key_status() 函数
* 是从串口中断中呼叫的，则此回调函数占用的将是中断的处理时间，与串口中断同级或优先级更低的中断
* 在返回前也不会得到执行，因此回调函数执行不宜占用过长的时间。
* 参数: pCallbackFunc  --  回调函数的函数指针
 ******************************************************************************/
void set_callback(void (*pCallbackFunc)(uint8_t));

/******************************************************************************
* 更新键盘状态 update_key_status()
* 输入参数为UART所接收数据，此函数一般可在UART中断处理中呼叫，但也可以根据需要从任何地方呼叫。
* 参数: RxData  --  UART从BC6xxx或BC759x所接收到的数据
******************************************************************************/
void update_key_status(uint8_t RxData);

/******************************************************************************
* 长按键计数 long_press_tick()
* 当希望进行长按键检测时，定期呼叫此函数，每呼叫此函数一次，则长按键计数加一。长按键的时间，
* 为用户通过 set_longpress_count() 设置的长按键计数目标值乘以呼叫此函数的时间间隔。此函数
* 一般在定时器中断中呼叫，但也可以根据需要从主程序循环或任何其它地方呼叫。
******************************************************************************/
void long_press_tick(void);

/******************************************************************************
* 获取当前键值 get_key_value()
* 可在任何时候呼叫。呼叫此函数后，如果当前 is_key_changed() 函数返回非零,呼叫后则返回值会变成
* 零。仅返回当前工作模式下有效的键值，即如果设置为不检测按键释放，则仅会返回按键按下所对应的键值。
* 返回值: 键值。如果当前按键是组合键或长按键，则返回用户给组合键和长按键自定义的键值。
******************************************************************************/
uint8_t get_key_value(void);



#endif
